part of flutter2web;

const String fileOption = "file";
const String defaultConfigFile = "flutter2web.yaml";

class Upload {
  AnsiPen greenPen = AnsiPen()..green();
  AnsiPen redPen = AnsiPen()..red();
  AnsiPen bluePen = AnsiPen()..blue();
  String server = 'https://flutter.su/appload';

  Encrypter _encrypter;
  int key_id;
  int version;
  int fileNumCheck = 0;
  int fileNumUpload = 0;
  Map<String,String> mapFiles;

  List<String> skipPathDir;
  List<String> skipPathFile;
  int substrDirName;


  start() async {
    print(greenPen('START flutter2web:upload'));
    
    var yamlConfig = File("flutter2web.yaml").existsSync() ? loadConfigFile("flutter2web.yaml") : loadConfigFile("pubspec.yaml");

    List<String> versionArray = yamlConfig['version'].split('+');

    if (versionArray.length < 2) throw redPen('ERROR version num!\nExample in pubspec.yaml, expect: version:0.0.1+1;');

    version = int.parse(versionArray[1]);



    String rsaPublicFileName = yamlConfig['flutter2web']['path_rsa_public'];
    final publicKey = await parseKeyFromFile<RSAPublicKey>(rsaPublicFileName);
    _encrypter = Encrypter(RSA(publicKey: publicKey));

    key_id = yamlConfig['flutter2web']['key_id'];

    if(yamlConfig['flutter2web']['server'] != null) server = yamlConfig['flutter2web']['server'];



    skipPathDir = [];
    skipPathFile = ['main.dart.js.map'];

    mapFiles = {};


    String dirName = yamlConfig['flutter2web']['path_build'];
    substrDirName = dirName.length + 1;

    _getFileList(dirName);

    Map<dynamic,dynamic> response = await _post({'method':'init'});
 
    if (response['status'] == 'error') {
      print(redPen('SERVER ERROR: ${response['message']}'));
    } else {
      var data = json.decode(response['data']);

      print(bluePen('INIT: ${data['result']['init']}'));

      response = await _post({'method':'checkFiles'}, mapFiles);

      data = json.decode(response['data']);

      print(bluePen('CHECK FILES:'));

      await data['result']['files'].forEach((key, value) async {
        if (value != 'equal') {
          var ret = await _post({'method':'sendFile', 'file': key}, '', File("$dirName/$key"));
          if(ret == null) print(redPen('- $key\n  [$value] -fail'));
          else print(bluePen('- $key\n  [$value]  +success'));
          fileNumUpload++;

          _checkUpload();
        } else {
          print(greenPen('- $key\n  [$value] =not modified'));
          fileNumUpload++;

          _checkUpload();
        }
      });
    }
  }// upload

  _getFileList(String dirName) {
    var dir = Directory(dirName);

    List contents = dir.listSync();

    String relativePath;
    String md5string;

    for (var fileOrDir in contents) {
      relativePath = fileOrDir.path.substring(substrDirName).replaceAll(RegExp(r"(\\)"), "/");

      if (fileOrDir is File) {
        if (skipPathFile.contains(relativePath)) continue;

        md5string = _md5file(fileOrDir);

        print('file: $relativePath [$md5string]');

        mapFiles[relativePath] = md5string;

        fileNumCheck++;
      } else if (fileOrDir is Directory) {

        if (skipPathDir.contains(relativePath)) continue;

        print('path: $relativePath');
        
        _getFileList(fileOrDir.path.replaceAll(RegExp(r"(\\)"), "/"));
      }
    }// for
  }


 _post(Map<dynamic,dynamic> signature, [data, File file]) async {
   signature['timestamp'] =  DateTime.now().toUtc().millisecondsSinceEpoch;
   signature['version'] =  version;

   final encrypted = _encrypter.encrypt(json.encode(signature));


   var uri = Uri.parse(server);
   var request = http.MultipartRequest("POST", uri);

   request.fields['key_id'] = "$key_id";
   request.fields['signature'] = "${encrypted.base64}";
   request.fields['data'] = data == null ? '' : (data.runtimeType == String ? data : json.encode(data));


   if (file != null) {
      var stream = http.ByteStream(DelegatingStream.typed(file.openRead()));
      var length = file.lengthSync();

      var multipartFile = http.MultipartFile('file', stream, length, filename: basename(file.path));

      request.files.add(multipartFile);
    }

    final responseString = await (await request.send()).stream.bytesToString();


    Map<String, dynamic> ret;

    try {
       ret = await json.decode(responseString);
    } catch(e) {
      print(redPen('ERROR RESPONSE: $responseString'));
    }

    return ret;
  }

  String _md5file(File source) {
    List<int> fileContent = source.readAsBytesSync();

    return md5.convert(fileContent).toString();
  }

  _checkUpload() async {
    if (fileNumCheck != fileNumUpload) return;

    var response = await _post({'method':'finishFiles'}, mapFiles);

    if (response['status'] == 'error') {
      print(redPen('SERVER ERROR: ${response['message']}'));
    } else {
      print(greenPen('END flutter2web:upload'));

      var data = json.decode(response['data']);

      print('OPEN URL: ${data['result']['url']}');
    }
  }
}
